/*
 * Copyright (C) 2013 Kaan Onarlioglu <onarliog@ccs.neu.edu>
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Kaan Onarlioglu -- www.onarlioglu.com
 * NEU SecLab      -- seclab.ccs.neu.edu
 *
 * utils.h, ver.2013.10.13
 *
 * Helper functions for Run-PrivExec.
 */

#ifndef UTILS_H
#define UTILS_H

/* Pretty print macros. */
#define NORMAL "\x1B[0m"
#define RED "\x1B[31m"
#define GREEN "\x1B[32m"

#define print_color( color, fmt, ...)               \
    do {                                            \
        printf( color fmt NORMAL, ##__VA_ARGS__);   \
        fflush( stdout);                            \
    } while ( 0)

/* Print in red. */
#define print_red( fmt, ...)                    \
    print_color( RED, fmt, ##__VA_ARGS__)

/* Print in green. */
#define print_green( fmt, ...)                  \
    print_color( GREEN, fmt, ##__VA_ARGS__)

/* Print in red with some error info. */
#define print_error( fmt, ...)                  \
    print_red( "Error in \"%s\": " fmt,         \
               __func__, ##__VA_ARGS__)

/* Exit codes. */
#define EXIT_PARENT     2
#define EXIT_FAILURE    1
#define EXIT_SUCCESS    0

/* Read this many bytes at once from files. */
#define IO_CHUNK 4096

/* Make these functions look like variables, because we can. */
#define TMP_DIR get_tmp_dir()
#define STORAGE_DIR get_storage_dir()
#define PRIVEXEC_DIR get_privexec_dir()
#define RANDOM_LEN 8
#define MAX_RANDOM_LEN 64
#define RANDOM_NAME get_random_name( RANDOM_LEN)

const char *get_privexec_dir( void);
const char *get_tmp_dir( void);
const char *get_storage_dir( void);
char *get_random_name( unsigned);

void set_cleanup_func( void ( *cleanup)( void));
void do_exit( int, char *, ...);
/* Shortcut for do_exit(). Use it. */
#define die( fmt, ...) do_exit( errno, fmt, ##__VA_ARGS__)

void do_strdup( char **, const char *);
void do_strcat( char **, const char *);
char *trim( char *);

char *read_text_file( char *, unsigned *);
void do_remove_dir_tree( char *);
void do_remove_dirs( char **);

void do_mount( char *, char *, char *, unsigned, char *);
void do_umount( char *);

void *try_realloc( void *, unsigned, unsigned *, unsigned, unsigned);
void do_free( void *);
void do_free_double( char **, unsigned);

#endif /* UTILS_H */
